
/* ******************************************************************************************************* */.                                    
                                                                                                                                                  
/* This is an example of how the macro can be called from SYNTAX    									                     */.                                    
/* Please copy the part between the lines of stars into your syntax 									                     */.                                    
/* Change the variables to the desired ones and change the path to the data file and to the macro file     */.                                    
/* in the new syntax - please put one exclamation mark in front of the macro name 		                     */.                                    
/* the macro name in your syntax should look like the name after the word DEFINE below                     */.                                    
                                                                                                                                                  
/* ******************************************************************************************************* */.

*Insert file='...\PATH TO THE DIRECTORY WHERE THE MACROS WERE SAVED... \mcr_SE_GrpPct_SPSS18.sps'.
*OMSEND.
*set mprint=no.

* GRPPCT 
       nrep = 80 /
       grp = IMMIG  /
       within = CNT  /
       wgt = W_FSTUWT /
       rwgt = W_FSTR /
       cons = 0.05 /
       miss = YES /
       PSU = SCHOOLID  /
       infile = '.../PATH TO THE DATA FILE/...' /.

/* ******************************************************************************************************** */.
/* ************ definition of the macro ******************************************************************* */.

set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

define !GRPPCT (nrep = !default(80) !charend('/')/
               within = !default(NOWITHIN) !charend('/')/
               grp = !charend('/')/
               wgt = !default(W_FSTUWT) !charend('/')/
               rwgt = !default(W_FSTR) !charend('/')/
               cons = !default(0.05) !charend('/')/
               PSU = !default(SCHOOLID) !charend('/')/
               limit_criteria = !default(0) !charend('/')/
               miss = !default(YES) !charend ('/')/
               infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers both TVars names TNumbers both.
omsend.

/* nWithin is the number of within group variables */
!if (!upcase(!within)="NOWITHIN") !then
!let !nWithin=!null
!Else
!do !w !in (!within)
!let !nwithin=!concat(!nWithin," ")
!doend !ifend.
!let !nwithin=!length(!nwithin).

get file !infile /keep !if (!nwithin>0) !then !within !ifend !grp !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep)).
!if (!upcase(!miss)="NO") !then
	!do !q !in ( !grp )
		select if (not missing(!q)).
	!doend.
	exe.
!ifend.
!if (!nwithin=0) !then compute !within = 1 !ifend.
sort cases by !within !grp.
save outfile='c:\temp\sort.sav'.


*** COMPUTE ESTIMATE ***.

weight by !wgt.
aggregate outfile=* mode=addvariables /presorted  /break=!within /N_all=n.
aggregate outfile=* /presorted /break=!within !grp /N_cases=n /N_all=mean(N_all) /NU_cases=NU.
formats N_cases (f10.0).
compute STAT=100*(n_cases/n_all).  
save outfile=!quote(!concat('c:\temp\',!wgt,'.sav')). 


* REPLICATES.   

!do !i= 1 !to !nrep.
dataset close ALL.
get file 'c:\temp\sort.sav'.
weight by !concat(!rwgt,!i).
aggregate outfile=* mode=addvariables /presorted  /break=!within /N_all=n.
aggregate outfile=* /presorted /break=!within !grp /N_grp=n /N_all=mean(N_all).
compute statr=100*(n_grp/n_all).  
save outfile=!quote(!concat('c:\temp\',!rwgt,!i,'.sav')).
!doend.


*** COMBINE RESULTS ***.

dataset close ALL.
/* get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')). */

!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.

sort cases by !within !grp.
match files file=* /table=!quote(!concat('c:\temp\',!wgt,'.sav')) /by !within !grp.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-STAT)**2.
aggregate outfile=*/ break=!within !grp NU_cases N_cases/ STAT= mean(STAT)/ var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute SE=sqrt(var).
formats STAT (f8.3)/ SE (f10.6).
sort cases by !within !grp.
save outfile='c:\temp\z.sav' /keep=!within !grp STAT se NU_cases N_cases.


*** ADD NUMBER OF SAMPLED SCHOOLS ***.

get file='c:\temp\sort.sav'.
sort cases by !within !grp !PSU.
weight off.
aggregate outfile=* /break=!within !grp !PSU /N=N.
aggregate outfile=* /break=!within !grp /NU_psu=N.
match files file='c:\temp\z.sav' /file=* /by !within !grp.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.

* Next part is different from manual.
*-----------------------------------------.
*!if (!length(!a)=3) !then
*aggregate /break=!if (!z=1) !then !head(!within)!else !within !ifend /N_total=sum(N_cases).
*compute FLAG_pct=((100*N_cases/N_total)<!b).
*formats FLAG_pct (f1.0).
*!ifend.
!doend.
*-----------------------------------------.

!ifend.
exe.


*** OUTPUT RESULTS ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

SUMMARIZE
  /TABLES=!if (!nwithin>0) !then !within !ifend !grp STAT se !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu !ifend
  /FORMAT=LIST NOCASENUM NOTOTAL 
  !if (!nwithin=0) !then /TITLE=!quote(!concat('SE on group percentages of ',!grp))
  !else /TITLE=!quote(!concat('SE on group percentages of ',!grp,' within ',!within,' categories')) !ifend
  /CELLS=NONE.
OMSEND TAG=["SumNoCaseProc"].

restore.
!enddefine.

