
/* ******************************************************************************************************* */.                                    
                                                                                                                                                  
/* This is an example of how the macro can be called from SYNTAX    									                     */.                                    
/* Please copy the part between the lines of stars into your syntax 									                     */.                                    
/* Change the variables to the desired ones and change the path to the data file and to the macro file     */.                                    
/* in the new syntax - please put one exclamation mark in front of the macro name 		                     */.                                    
/* the macro name in your syntax should look like the name after the word DEFINE below                     */.                                    
                                                                                                                                                  
/* ******************************************************************************************************* */.

*Insert file='...\PATH TO THE DIRECTORY WHERE THE MACROS WERE SAVED... \mcr_SE_PctLev_SPSS18.sps'.
*OMSEND.
*set mprint=no.

* PCTLEV 
       nrep = 80 /
       proflev = PL_PV1READ PL_PV2READ PL_PV3READ PL_PV4READ PL_PV5READ /
       within = CNT  /
       wgt = W_FSTUWT /
       rwgt = W_FSTR /
       cons = 0.05 /
       PSU = SCHOOLID  /
       infile = '.../PATH TO THE DATA FILE/...' /.

/* ******************************************************************************************************** */.
/* ************ definition of the macro ******************************************************************* */.

SET mprint=no.

*** Created by Alexander Daraganov & Eveline Gebhardt, Australian Council for Educational Research ***.

define !PCTLEV (nrep = !default(80) !charend('/')/
               within = !default(NOWITHIN) !charend('/') /
               proflev = !charend('/') /
               wgt = !default(W_FSTUWT) !charend('/') /
               rwgt =  !default(W_FSTR) !charend('/') /
               cons = !default(0.05) !charend('/')/
               PSU = !default(SCHOOLID) !charend('/')/
               limit_criteria = !default(0) !charend('/')/
               infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers both TVars names TNumbers both.
omsend.

/* nWithin is the number of within group variables */
!if (!upcase(!within)="NOWITHIN") !then
!let !nWithin=!null
!Else
!do !w !in (!within)
!let !nwithin=!concat(!nWithin," ")
!doend !ifend.
!let !nwithin=!length(!nwithin).
!let !zz=!blanks(0).
!let !nu=!blanks(0).
*** COMPUTE ESTIMATE ***.
!do !j !in (!proflev).
!let !zz=!concat(!zz,!blanks(1)).
!let !zzz=!concat('NU_cases_',!length(!zz)).
!let !nu=!concat(!nu,!zzz,',').
get file !infile /keep !if (!nwithin>0) !then !within !ifend !j !PSU !wgt.
!if (!nwithin=0) !then compute !within = 1 !ifend .
rename vars (!j=proflev).
sort cases by !within proflev .
weight by !wgt.
aggregate outfile=!quote(!concat('c:\temp\',!j,'.sav')) /presorted /break=!within proflev /!j=n  /N_cases=n /!zzz=NU .
!doend.
!let !nu=!substr(!nu,1,!length(!substr(!nu,2))).
match files !do !j !in (!proflev) /file=!quote(!concat('c:\temp\',!j,'.sav')) !doend /by !within proflev.
compute NU_cases=Min(!nu).
aggregate outfile=* mode=addvariables /break=!within/N_all=sum(!head(!proflev)) .


do repeat s=stat1 to stat5 /p=!proflev.
compute s=100*p/N_all.
End repeat.
save outfile=!quote(!concat('c:\temp\',!wgt,'.sav'))  /* /keep=!within proflev stat1 to stat5. */.


* REPLICATES.   

!do !i= 1 !to !nrep.
!do !j !in (!proflev).
dataset close ALL.
get file !infile /keep !if (!nwithin>0) !then !within !ifend !j !concat(!rwgt,!i).
!if (!nwithin=0) !then compute !within = 1 !ifend .
rename vars (!j=proflev).
sort cases by !within proflev .
weight by !concat(!rwgt,!i).
aggregate outfile=!quote(!concat('c:\temp\',!i,!j,'.sav')) /presorted /break=!within proflev /!j=n.
!doend.

match files !do !j !in (!proflev) /file=!quote(!concat('c:\temp\',!i,!j,'.sav')) !doend /by !within proflev.
aggregate outfile=* mode=addvariables /break=!within /N_all=sum(!head(!proflev)).

do repeat s=statR1 to statR5 /p=!proflev.
compute s=100*p/N_all.
End repeat.

save outfile=!quote(!concat('c:\temp\',!rwgt,!i,'.sav')) /keep=!within proflev statR1 to statR5..
!doend.


*** COMBINE RESULTS ***.

dataset close ALL.
/* get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')). */

!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.

sort cases by !within proflev.

match files file=* /table=!quote(!concat('c:\temp\',!wgt,'.sav')) /by !within proflev.
recode stat1 to stat5 statR1 to statR5 (sysmis=0).
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

do repeat a=statr1 to statr5/
	  b=stat1 to stat5/
	  c=var1 to var5.
compute c=(a-b)**2.
end repeat.

aggregate outfile=*/
	  break=!within proflev NU_cases N_cases/ 										
          stat1 to stat5= mean(stat1 to stat5)/
          var1 to var5 = sum(var1 to var5).

do repeat a=var1 to var5.
compute a=!cons*a.
end repeat.

compute pv_var=mean(var1 to var5).



*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute STAT=mean(stat1 to stat5).

do repeat a=stat1 to stat5/b=pvar1 to pvar5.
compute b=(a-STAT)**2.
end repeat.

compute pvmerr=.25*(sum(pvar1 to pvar5)).



*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
formats STAT (f8.3)/ SE (f10.6).
sort cases by !within proflev.
save outfile='c:\temp\z.sav' /keep=!within proflev STAT se NU_cases N_cases.



*** ADD NUMBER OF SAMPLED SCHOOLS ***.
!let !zz=!null.
!let !zzz=!null.
!let !nu=!null.
!let !nu_del=!null.
!let !rec=!null
!do !j !in (!proflev).
!let !zz=!concat(!zz,!blanks(1)).
!let !zzz=!concat('NU_psu_',!length(!zz)).
!let !rec=!concat(!rec,'/',!zzz,'=N(',!j,') ').
!let !nu=!concat(!nu, !zzz, ',').
!let !nu_del=!concat(!nu, !zzz, !blanks(1)).
dataset close ALL.
get file !infile /keep !if (!nwithin>0) !then !within !ifend !j !concat(!rwgt,!i) !PSU.
!if (!nwithin=0) !then compute !within = 1 !ifend .
rename vars (!j=proflev).
sort cases by !within proflev !PSU.
weight off.
aggregate outfile=!quote(!concat('c:\temp\',!j,'.sav')) /presorted /break=!within proflev !PSU/!j=N.
!doend.
!let !nu=!substr(!nu, 1,!length(!substr(!nu,2))).
match files !do !j !in (!proflev) /file=!quote(!concat('c:\temp\',!j,'.sav')) !doend /by !within proflev !PSU.
aggregate outfile=* /break=!within proflev !rec.
compute NU_psu=min(!nu).
exe.
delete vars !nu_del .
match files file='c:\temp\z.sav' /file=* /by !within proflev.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.

* Next part is different from manual.
*-----------------------------------------.
*!if (!length(!a)=3) !then
*aggregate /break=!if (!z=1) !then !head(!within)!else !within !ifend /N_total=sum(N_cases).
*compute FLAG_pct=((100*N_cases/N_total)<!b).
*formats FLAG_pct (f1.0).
*!ifend.
*-----------------------------------------.

!doend.
!ifend.
exe.


*** OUTPUT TABLE ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

SUMMARIZE
  /TABLES=!if (!nwithin>0) !then !within !ifend proflev STAT se !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT=LIST NOCASENUM NOTOTAL 
  !if (!nwithin=0) !then /TITLE='SE on percentages in proficiency levels'
  !else /TITLE=!quote(!concat('SE on percentages in proficiency levels by ',!within)) !ifend
  /CELLS=NONE.
OMSEND TAG=["SumNoCaseProc"].

delete variables !if(!upcase(!within)=NOWITHIN) !then !within !ifend .

!enddefine.