
/* ******************************************************************************************************* */.                                    
                                                                                                                                                  
/* This is an example of how the macro can be called from SYNTAX    									                     */.                                    
/* Please copy the part between the lines of stars into your syntax 									                     */.                                    
/* Change the variables to the desired ones and change the path to the data file and to the macro file     */.                                    
/* in the new syntax - please put one exclamation mark in front of the macro name 		                     */.                                    
/* the macro name in your syntax should look like the name after the word DEFINE below                     */.                                    
                                                                                                                                                  
/* ******************************************************************************************************* */.
*Insert file='...\PATH TO THE DIRECTORY WHERE THE MACROS WERE SAVED... \mcr_SE_reg_SPSS18.sps'.
*OMSEND.
*set mprint=no.

* RegNoPV
       nrep = 80 /
       dep = joyread /
       ind = IMMIG  /
       grp = CNT  /
       wgt = W_FSTUWT /
       rwgt = W_FSTR /
       cons = 0.05 /
       PSU = SCHOOLID  /
       infile = '.../PATH TO THE DATA FILE/...' /.

/* ******************************************************************************************************** */.
/* ************ definition of the macro ******************************************************************* */.

set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

define !REGnoPV (nrep = !default(80) !charend('/')/
                ind = !charend('/') /
                dep = !charend('/') /
                grp = !default(NOGRP) !charend('/') /
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                cons = !default(0.05) !charend('/')/
                PSU = !default(SCHOOLID) !charend('/')/
                limit_criteria = !default(0) !charend('/')/
                infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!if (!upcase(!grp)=NOGRP) !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
title !quote(!concat('Number of group variables = ',!ngrp)).

get file !infile /keep=!if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !dep !ind.
!if (!ngrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
sort cases by !grp.       
compute MISS = (missing(!dep)).
!do !i !in (!ind).
if (missing(!i)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE ESTIMATES ***.
     
get file 'c:\temp\sort.sav'.
split file by !grp.            
weight by !wgt.

oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2.sav' /TAG="R2".
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB('C:\temp\coef.sav') .
omsend tag="R2".

get file='C:\temp\coef.sav'.
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
VARSTOCASES  /MAKE STAT FROM @INTERCEPT !ind
 /INDEX = ind(STAT) /KEEP = !grp /NULL = KEEP.
ALTER TYPE  IND(A20).
Recode ind ('@INTERCEPT'=' INTERCEPT').
save outfile='c:\temp\all.sav'.

get file='c:\temp\R2.sav' /keep=!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp STAT).
formats STAT (f8.2).
string IND (a20).
compute IND="  R-SQUARE".
add files file='c:\temp\all.sav' /file=*.
sort cases by !grp ind.
save outfile='c:\temp\all2.sav'.


* REPLICATES.
oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2R.sav' /TAG="R2R".
!do !i=1 !to !nrep.
dataset close all.
get file='c:\temp\sort.sav'.

*sort cases by !grp.            
split file by !grp.            
                              
weight by !concat(!rwgt,!i).
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB('C:\temp\coef.sav') .

get file='C:\temp\coef.sav'.
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
VARSTOCASES  /MAKE statR FROM @INTERCEPT !ind
 /INDEX = ind(statR) /KEEP = !grp /NULL = KEEP.
ALTER TYPE  IND(A20).
Recode ind ('@INTERCEPT'=' INTERCEPT').

save outfile=!quote(!concat('C:\temp\',!rwgt,!i,'.sav')).
!doend.
omsend tag="R2R".


*** COMBINE RESULTS ***.

dataset close ALL.
/* get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')). */

!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.
save outfile='c:\temp\rwgt.sav'.

get file='c:\temp\R2R.sav' /keep=!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp statR).
formats statr (f8.2).
string IND (a20).
compute IND="  R-SQUARE".
add files file='c:\temp\rwgt.sav' /file=*.
sort cases by !grp ind.
save outfile='c:\temp\rwgt2.sav'.

sort cases by !grp ind.
match files file=* /table='c:\temp\all2.sav' /by !grp ind.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-STAT)**2.

aggregate outfile=* /break=!grp IND /STAT= mean(STAT) /var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute se=sqrt(var).
formats STAT (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
get file='c:\temp\sort.sav'.
weight by !wgt.
aggregate /break=!if (!z=1) !then !head(!grp) !else !grp !ifend /N_total=N.
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N /N_total=mean(N_total).
aggregate outfile=* /break=!grp /NU_psu=N /N_total=mean(N_total).
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
filter off.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.

* Next part is different from manual.
*-----------------------------------------.
*!if (!length(!a)=3) !then
*compute FLAG_pct=((100*N_cases/N_total)<!b).
*formats FLAG_pct (f1.0) N_total (f10.0).
*!ifend.
!doend.
exe.
delete var N_total.
*-----------------------------------------.

!ifend.
compute IND=Ltrim(IND).
exe.
omsend tag="crap".

*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp>0) !then !grp !ifend IND STAT SE !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Regression of ",!dep," on ",!ind)) 
  !else /TITLE=!quote(!concat("Regression of ",!dep," on ",!ind," by ",!grp)) !ifend
  /CELLS=NONE.

delete variables !if (!limit_criteria=0) !then N_total !ifend !if (!ngrp=0) !then !grp !ifend  var.
omsend tag="case".
restore.
!enddefine.


