
/* ******************************************************************************************************* */.                                    
                                                                                                                                                  
/* This is an example of how the macro can be called from SYNTAX    									                     */.                                    
/* Please copy the part between the lines of stars into your syntax 									                     */.                                    
/* Change the variables to the desired ones and change the path to the data file and to the macro file     */.                                    
/* in the new syntax - please put one exclamation mark in front of the macro name 		                     */.                                    
/* the macro name in your syntax should look like the name after the word DEFINE below                     */.                                    
                                                                                                                                                  
/* ******************************************************************************************************* */.                                    
*Insert file='...\PATH TO THE DIRECTORY WHERE THE MACROS WERE SAVED... \mcr_SE_univ_SPSS18.sps'.
*OMSEND.
*set mprint=no.

* UNIVAR                                                                                              
      nrep = 80 /                                                                                     
      stat = stddev  /                                                                               
      dep = SCHSIZE  /                                                                               
      grp = CNT  /                                                                                   
      wgt = W_FSTUWT /                                                                               
      rwgt = W_FSTR /                                                                                
      cons = 0.05 /                                                                                  
      PSU = SCHOOLID  /                                                                              
      infile = '.../PATH TO THE DATA FILE/...' .                                                           
                                                                                                   
/* ******************************************************************************************************** */.
/* ************ definition of the macro ******************************************************************* */.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

DEFINE !UNIVAR (nrep = !default(80) !charend('/')/
               STAT = !charend('/')/
               dep = !charend('/')/
               grp = !default(NOGRP) !charend('/')/
               wgt = !default(W_FSTUWT) !charend('/')/
               rwgt = !default(W_FSTR) !charend('/')/
               cons = !default(0.05) !charend('/')/
               PSU = !default(SCHOOLID) !charend('/')/
               limit_criteria = !default(0) !charend('/')/
               infile = !charend('/')).
preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables TEXTS /destination viewer=no /Tag="NoViewer".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null.
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
!if (!ngrp=0) !then 
!let !x=2.
!else.
/* x is nGrp + 1 */
!let !x=!length(!concat(!blanks(!nGrp)," ")).
!ifend.
title !concat('(number of group variables is ',!nGrp,')(nGrp + 1 = ',!x,')').

dataset close all.
GET FILE !infile 
   /KEEP !if(!nGrp>0) !then !grp !ifend !PSU !dep !wgt !do !i= 1 !to !nrep !concat(!rwgt,!i) !doend.
!if(!nGrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
fre !grp.
SORT CASES BY !grp.
compute MISS=0.
!do !d !in (!dep).
if (missing(!d)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\a.sav' /drop=MISS.

*** COMPUTE ESTIMATE ***.

oms /select tables /if subtypes='Report' /destination format=sav outfile='c:\temp\fullwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='FullWeight'.
WEIGHT BY !wgt.
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
omsend tag=['FullWeight'].


* REPLICATES.   

oms /select tables /if subtypes='Report' /destination format=sav numbered="RepWeight" outfile='c:\temp\repwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='RepWeight'.
!do !i= 1 !to !nrep.
WEIGHT BY !concat(!rwgt,!i).
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
!doend.
omsend tag=['RepWeight'].


*** COMBINE FILES ***.

!if (!ngrp=0) !then !let !ngrp=1 !ifend.

get file='c:\temp\repwgt.sav' .
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).
rename vars (!dep=!do !d !in (!dep) !concat("R_",!d) !doend).
save outfile='c:\temp\repwgt2.sav'.

get file='c:\temp\fullwgt.sav' .
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).

match files table=* /file='c:\temp\repwgt2.sav' /by !do !n=1 !to !nGrp !concat(" Var",!n) !doend !concat(" Var",!x).
exe.

rename vars (!concat("Var",!x)=statistic).
!do !s !in (!STAT)
!if (!upcase(!s)=MEAN) !then recode statistic ("Mean"="MEAN") !ifend.
!if (!upcase(!s)=STDDEV) !then recode statistic("Std. Deviation"="STDDEV") !ifend.
!if (!upcase(!s)=MEDIAN) !then recode statistic("Median"="MEDIAN") !ifend.
!if (!upcase(!s)=SKEW) !then recode statistic("Skewness"="SKEW") !ifend.
!if (!upcase(!s)=VARIANCE) !then recode statistic("Variance"="VARIANCE") !ifend.
!if (!upcase(!s)=KURT) !then recode statistic("Kurtosis"="KURT") !ifend.
!doend.

!if (!nGrp > 0) !Then
rename vars (!do !n=1 !to !nGrp !concat("Var",!n) !doend =!grp).
!Ifend.


*** COMPUTE SAMPLING VARIANCE (U) ***.

!do !d !in(!dep) 
COMPUTE !concat('var_',!d) = !cons*((!concat('r_',!d) - !d)**2).   
!doend.

AGGREGATE OUTFILE =*
   /PRESORTED /BREAK=!if (!nGrp > 0) !then !grp !ifend statistic 
   /!do !d !in(!dep) !d !doend=MEAN(!do !d !in(!dep) !d !doend) 
   /!do !d !in(!dep) !concat('var_',!d) !doend=SUM(!do !d !in(!dep) !concat('var_',!d) !doend) .


*** COMPUTE SE ***.

!do !d !in(!dep) 
COMPUTE !concat('se_',!d) = SQRT(!concat('var_',!d) ).
FORMATS !d (F8.3) / !concat('se_',!d) (F10.6).
!doend .


*** CREATE MERGE VARIABLE ***.

* Note: This might not work in all languages .
!do !g !in (!grp).
select if (substr(!g,1,3)<>"Tot").
compute !g=rtrim(!g).
!doend.
sort cases by statistic !grp.
compute z=$casenum.
autorecode statistic /into temp.
aggregate /break=temp /max=max(z) /N=N.
compute y=max-z.
compute x=mod(y,z).
do if (temp=1).
compute MERGE=z.
else.
compute MERGE=N-x.
end if.
SORT CASES BY MERGE.
save outfile='c:\temp\z.sav' /DROP=TEMP X Y Z.


*** COMPUTE N ***.

get file='c:\temp\a.sav'.
weight off.
aggregate /break=!if (!z=1) !then !head(!grp)!else !grp !ifend /N_total=sum(!wgt).
filter by VALIDN.
weight by !wgt.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N /N_total=mean(N_total).
aggregate outfile=* /break=!grp /NU_psu=N /N_total=mean(N_total).
filter off.

match files /file='c:\temp\y.sav' /file=* /by !grp.
Compute MERGE=$casenum.
exe.
delete var !grp.
match files file='c:\temp\z.sav' /TABLE=* /by MERGE.
formats N_cases (f10.0).
exe.
omsend Tag="NoViewer".


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then delete var N_total !else
!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.

* Next part is different from manual.
*-----------------------------------------.
*!if (!length(!a)=3) !then
*compute FLAG_pct=((100*N_cases/N_total)<!b).
*formats FLAG_pct (f1.0) N_total (f10.0).
*!ifend.
!doend.
exe.
delete var N_total.
*-----------------------------------------.

!ifend.
exe.


*** OUTPUT TABLES ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".
!do !s !in (!STAT).
Temp.
Select if (statistic=!quote(!upcase(!s))).

SUMMARIZE
  /TABLES= !if (!upcase(!grp)="NOGRP") !then !else !grp !ifend statISTIC !do !d !in(!dep) !d !concat('SE_',!d) !doend 
  !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT= !if (!nGrp > 0) !then LIST NOCASENUM TOTAL !else LIST NOCASENUM !ifend
  !if (!nGrp=0) !then /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!dep))
  !else /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!dep,' by ',!grp)) !ifend
  /CELLS=NONE.
!doend 

OMSEND TAG=["SumNoCaseProc"].

delete variables !if (!upcase(!grp)="NOGRP") !then !grp !ifend !do !d !in(!dep) !concat(" var_",!d) !doend MERGE max N.
set mprint=no.
restore.
!enddefine.